#!/usr/bin/python3

import openpyxl, yaml, time
from openpyxl.styles import Alignment

class GridLog:

    def __init__(self, cfg):
        self.cfg = yaml.load(cfg) # get config
        logstamp = time.strftime('%Y-%m-%d_%H-%M-%S') # timestamp for text aux log files
        for workbook in self.cfg:
            self.cfg[workbook]['GLworkbook'] = openpyxl.Workbook() # create a workbook object
            summarySheet = self.cfg[workbook]['GLworkbook'].worksheets[0] # set default sheet name to summary
            summarySheet.title = 'Summary'
            summarySheet.append(('Tab', 'Description')) # summary of tabs and what they are
            for tab in self.cfg[workbook]['tabs']: # set up tabs defined in cfg
                descrip = self.cfg[workbook]['tabs'][tab].get('description', '') # set up description on summary tb
                self.cfg[workbook]['tabs'][tab]['rowheaders'] = self.cfg[workbook]['tabs'][tab].get('rowheaders', 0) # set defaults for headers
                self.cfg[workbook]['tabs'][tab]['colheaders'] = self.cfg[workbook]['tabs'][tab].get('colheaders', 0)
                newSheet = self.cfg[workbook]['GLworkbook'].create_sheet(title = tab) # make the new sheet
                summarySheet.append((tab, descrip)) # add the tab name and description to the summary tab
                summarySheet[f'A{summarySheet.max_row}'].alignment = Alignment(wrapText = True, vertical = 'top', horizontal = 'left') # fix alignments
                summarySheet[f'B{summarySheet.max_row}'].alignment = Alignment(wrapText = True, vertical = 'top', horizontal = 'left')
                self.cfg[workbook]['tabs'][tab]['logfile'] = self.cfg[workbook]['tabs'][tab].get('logfile') # set default logfile to none
                # set column titles
                numCols = self.cfg[workbook]['tabs'][tab]['colheaders'] # support multi row headers
                for col in self.cfg[workbook]['tabs'][tab]['cols']: # set header on each column
                    self.cfg[workbook]['tabs'][tab]['cols'][col]['title'] = self.cfg[workbook]['tabs'][tab]['cols'][col].get('title', '')
                    titles = self.cfg[workbook]['tabs'][tab]['cols'][col]['title'].split('|') # multi row headers are split on | character
                    while len(titles) < numCols: # force titles to at least as many rows as headers are needed
                        titles.append('')
                    for i in range(1, numCols + 1):
                        newSheet[f'{col}{i}'] = titles[i - 1]
                if self.cfg[workbook]['tabs'][tab]['logfile']: # open up log files, one log file per tab
                    self.cfg[workbook]['tabs'][tab]['GLlogfile'] = open(f'{self.cfg[workbook]["tabs"][tab]["logfile"]}_{logstamp}.log', 'wt')
                else:
                    self.cfg[workbook]['tabs'][tab]['GLlogfile'] = None

    def addRow(self, workbook, tab, rowdata): # main logging function
        sheet = self.cfg[workbook]['GLworkbook'].get_sheet_by_name(tab) # find tab to post to
        sheet.append(rowdata) # write the data, bump max_row
        for c in range(len(rowdata)):
            cell = f'{chr(0x41 + c)}{sheet.max_row}'
            sheet[cell].alignment = Alignment(wrapText = True, vertical = 'top', horizontal = 'left') # set alignment
        logFile = self.cfg[workbook]['tabs'][tab]['GLlogfile']
        if logFile: # if this tab has a text log file, write entry with field name and time stamp
            timeStamp = time.strftime('%Y-%m-%d %H:%M:%S')
            for l in range(len(rowdata)):
                outStr = rowdata[l].strip()
                colLabel = chr(0x41 + l)
                title = None
                if colLabel in self.cfg[workbook]['tabs'][tab]['cols']:
                    title = self.cfg[workbook]['tabs'][tab]['cols'][colLabel].get('title')
                if title:
                    logFile.write(f'{timeStamp} {title}: {outStr}\n')
                else:
                    logFile.write(f'{timeStamp} {outStr}\n')

    def close(self): # save workbooks, close logs
        heightPerLine = 15
        widthPerChar = 1.25
        for workbook in self.cfg: # freeze panes, set column widths and row heights
            for tab in self.cfg[workbook]['GLworkbook'].sheetnames:
                sheet = self.cfg[workbook]['GLworkbook'].get_sheet_by_name(tab)
                # set column widths and heights- assume 1.1 for width of character, 15 height per line
                for rowidx, row in enumerate(sheet.rows): # get row height
                    rowHeight = heightPerLine
                    for cell in row:
                        if cell.value:
                            rowHeight = max(rowHeight, len(cell.value.split('\n')) * heightPerLine)
                    rowHeight = max(15, rowHeight)
                    sheet.row_dimensions[rowidx + 1].height = rowHeight
                for colidx, col in enumerate(sheet.columns): # get column width
                    colWidth = widthPerChar
                    for cell in col:
                        if cell.value:
                            for l in cell.value.split('\n'):
                                colWidth = max(colWidth, len(l) * widthPerChar)
                    sheet.column_dimensions[chr(0x41 + colidx)].width = colWidth
                    print(f'tab {tab}: width: {colWidth} height: {rowHeight}')
                if tab != 'Summary':
                    rowhead = self.cfg[workbook]['tabs'][tab]['rowheaders']
                    colhead = self.cfg[workbook]['tabs'][tab]['colheaders']
                    if rowhead or colhead:
                        sheet.freeze_panes = sheet[f'{chr(0x41 + rowhead)}{colhead + 1}']
            summarySheet = self.cfg[workbook]['GLworkbook'].get_sheet_by_name('Summary')
            summarySheet.freeze_panes = summarySheet['A2'] # freeze top row
            self.cfg[workbook]['GLworkbook'].save(workbook)
            for tab in self.cfg[workbook]['tabs']:
                if self.cfg[workbook]['tabs'][tab]['GLlogfile']:
                    self.cfg[workbook]['tabs'][tab]['GLlogfile'].close()

if __name__ == '__main__':
    from pprint import pprint
    import re
    cfg = """
---
test.xlsx:
    tabs:
        Systemd:
            logfile: systemd # log file for this tab
            description: >
                Systemd messages from /var/log/messages
            rowheaders: 0
            colheaders: 1
            cols:
                A:
                    title: Timestamp
                B:
                    title: PID
                C:
                    title: Systemd message
        Secure:
            logfile: securelog
            description: >
                Log entries found in /var/log/secure
            rowheaders: 0
            colheaders: 1
            cols:
                A:
                    title: Timestamp
                B:
                    title: PID
                C:
                    title: Secure log message
        Cron:
            logfile: cronlog
            description: >
                Cron logging, taken from /var/log/cron
            rowheaders: 0
            colheaders: 1
            cols:
                A:
                    title: Timestamp
                B:
                    title: PID
                C:
                    title: Cron log message
    """
    logLineRe = re.compile(r'^([a-z]{3}\s+\d+\s+\d+:\d+:\d+)\s+linux\s+([^\[]+)\[(\d+)\]:\s+(.*)$', re.I)
    gLog = GridLog(cfg)
    for logger in zip(('/var/log/messages', '/var/log/secure', '/var/log/cron'), ('systemd', None, None), ('Systemd', 'Secure', 'Cron')):
        for l in open(logger[0], 'rt'):
            m = logLineRe.match(l)
            if m:
                if logger[1]:
                    if m.group(2) != logger[1]: continue
                gLog.addRow('test.xlsx', logger[2], (m.group(1), m.group(3), m.group(4)))
    gLog.close()
